from django.contrib import admin
from .grepesjs_mixin import GrapesJsAdminMixin
from garpix_page.models import GrapesJsHtmlComponent
from .base_component import BaseComponentAdmin


@admin.register(GrapesJsHtmlComponent)
class GrapesJsHtmlComponentAdmin(BaseComponentAdmin, GrapesJsAdminMixin):
    pass



from rest_framework.fields import ReadOnlyField
from rest_framework.serializers import ModelSerializer

from garpix_page.utils.get_exclude_fields import get_exclude_fields


def get_serializer(model):
    if model.get_serializer(model) is not None:
        return model.get_serializer(model)

    return type(f'{model.__name__}Serializer', (ModelSerializer, ), {
        'seo_title': ReadOnlyField(source='get_seo_title'),
        'seo_keywords': ReadOnlyField(source='get_seo_keywords'),
        'seo_description': ReadOnlyField(source='get_seo_description'),
        'seo_author': ReadOnlyField(source='get_seo_author'),
        'seo_og_type': ReadOnlyField(source='get_seo_og_type'),
        'Meta': type('Meta', (object,), {
            'model': model,
            'exclude': ('sites', 'lft', 'rght', 'tree_id', 'level', 'polymorphic_ctype', 'parent',) + tuple(get_exclude_fields(model))
            # 'fields': '__all__'
        })
    })


def get_components_serializer(model):
    if model.get_serializer(model) is not None:
        return model.get_serializer(model)

    return type(f'{model.__name__}Serializer', (ModelSerializer, ), {
        'Meta': type('Meta', (object,), {
            'model': model,
            'exclude': ('polymorphic_ctype', 'pages') + tuple(get_exclude_fields(model))
        })
    })



import re
from functools import reduce
from django.template import Template, Context
from garpix_page.settings import NAME_RENDER_TAG

__all__ = ('ApplyRenderTag', )


REGEX_RENDER_TAG = '<%s>(.*?)</%s>' % (NAME_RENDER_TAG, NAME_RENDER_TAG)


class ApplyRenderTag(object):
    def apply_tag_init(self, string):
        strings_to_render = re.findall(REGEX_RENDER_TAG, string)
        replace_to_strings = map(lambda t: t.render(Context({})), map(Template, strings_to_render))

        return reduce(lambda s, r: re.sub(REGEX_RENDER_TAG, r, s, 1), replace_to_strings, string)

    def apply_tag_save(self, string):
        return string



from django.core.management.base import SystemCheckError
from django.db import models

from garpix_page.forms.widgets import GrapesJsWidget
from garpix_page.forms.fields import GrapesJsField
from garpix_page.settings import BASE, GRAPESJS_DEFAULT_HTML, REDACTOR_CONFIG

__all__ = (
    'GrapesJsHtmlField',
)


class GrapesJsHtmlField(models.TextField):
    """
    Model field with support grapesjs.

    """
    def __init__(self, default_html=GRAPESJS_DEFAULT_HTML, redactor_config=BASE,
                 apply_django_tag=False, validate_tags=False, template_choices=None, **kwargs):
        if kwargs.get('choices'):
            raise ValueError(
                "use 'template_choices' instead of 'choices' in the '%s'" % self.__class__.__name__
            )

        if self.check_template_choices(template_choices):
            default_html = template_choices[0][0]

        self.params_for_formfield = {
            'default_html': default_html,
            'html_name_init_conf': REDACTOR_CONFIG[redactor_config],
            'apply_django_tag': apply_django_tag,
            'validate_tags': validate_tags,
            'template_choices': template_choices,
            'form_class': GrapesJsField,
            'widget': GrapesJsWidget,
        }

        super().__init__(**kwargs)

    def formfield(self, **kwargs):
        return super().formfield(**{**kwargs, **self.params_for_formfield})

    def check_template_choices(self, value):
        if not value:
            return False

        self.choices = value
        error = self._check_choices()

        if error:
            error_info = error[0].msg.replace('choices', 'template_choices')
            raise SystemCheckError('\nERROR:\n%s: %s' % (self.__class__.__name__, error_info, ))

        return True



from garpix_page.models import GarpixPageSiteConfiguration


from django.contrib.sitemaps import GenericSitemap
from garpix_page.models.base_page import BasePage


def sitemap_view():
    pages = {
        'queryset': BasePage.on_site.filter(is_active=True).order_by('title'),
        'date_field': 'created_at'
    }
    extra_data = {
        'priority': 1.0
    }
    try:
        config = GarpixPageSiteConfiguration.get_solo()
        extra_data.update({
            'changefreq': config.sitemap_frequency
        })
    except Exception:
        pass

    return {
        'sitemaps': {
            'pages': GenericSitemap(
                pages,
                **extra_data
            ),
        }
    }



from django.contrib import admin
from django.contrib.admin.widgets import FilteredSelectMultiple
from django.utils.text import format_lazy
from modeltranslation.admin import TabbedTranslationAdmin
from polymorphic.admin import PolymorphicParentModelAdmin, PolymorphicChildModelFilter, PolymorphicChildModelAdmin

from django.urls import reverse
from django.http import HttpResponseRedirect
from django.urls import path
from garpix_page.models import BaseComponent, BasePage
from garpix_page.utils.get_garpix_page_models import get_garpix_page_component_models
from ..forms import PolymorphicModelPreviewChoiceForm


class BaseComponentAdmin(PolymorphicChildModelAdmin, TabbedTranslationAdmin):
    base_model = BaseComponent
    list_display = ('title', 'model_name')
    search_fields = ('title', 'pages__title')

    filter_horizontal = (
        'pages',
    )

    change_form_template = 'garpix_page/admin/component_change_form.html'

    def formfield_for_manytomany(self, db_field, request, **kwargs):

        if db_field.name == 'pages':
            kwargs['widget'] = FilteredSelectMultiple(
                db_field.verbose_name, is_stacked=False
            )
        else:
            return super().formfield_for_manytomany(db_field, request, **kwargs)
        if 'queryset' not in kwargs:
            queryset = BasePage.objects.all()
            if queryset is not None:
                kwargs['queryset'] = queryset
        form_field = db_field.formfield(**kwargs)
        msg = 'Hold down “Control”, or “Command” on a Mac, to select more than one.'
        help_text = form_field.help_text
        form_field.help_text = (
            format_lazy('{} {}', help_text, msg) if help_text else msg
        )
        return form_field

    def has_module_permission(self, request):
        return False

    def get_form(self, request, obj=None, **kwargs):
        if request.GET.get('_popup') and request.GET.get('_to_field'):
            self.exclude = ('pages',)
        return super().get_form(request, obj=None, **kwargs)


@admin.register(BaseComponent)
class RealBaseComponentAdmin(PolymorphicParentModelAdmin, TabbedTranslationAdmin):
    child_models = get_garpix_page_component_models()
    base_model = BaseComponent
    list_filter = (PolymorphicChildModelFilter, 'pages')
    add_type_form = PolymorphicModelPreviewChoiceForm
    save_on_top = True
    list_display = ('title', 'pages_list', 'model_name', 'is_active')
    search_fields = ('title', 'pages__title')
    list_editable = ('is_active',)
    actions = ('clone_object', )

    def pages_list(self, obj):
        pages = obj.pages.all()
        pages_str = ', '.join(pages[:6].values_list('title', flat=True))
        if (more_count := pages.count() - 6) > 0:
            pages_str += f' ...еще {more_count}'
        return pages_str

    pages_list.short_description = 'Страницы для отображения'

    def clone_object(self, request, queryset):
        """Копирование(клонирование) выбранных объектов - action"""
        for obj in queryset:

            obj = obj.get_real_instance()

            len_old_title = obj.__class__.objects.filter(title__icontains=obj.title).count()
            title = f"{obj.title} ({len_old_title})" if len_old_title > 0 else obj.title

            new_obj = obj.clone_object(title=title)

            new_obj.save()

    clone_object.short_description = 'Клонировать объект'

    def get_urls(self):
        urls = super().get_urls()

        my_urls = [
            path('<int:pk>/change/full_clone/', self.full_clone, name='full_clone/'),
        ]

        return my_urls + urls

    def full_clone(self, request, pk):
        if request.method == 'POST':
            obj = self.get_object(request, pk)

            obj = obj.get_real_instance()

            title = request.POST.get('title', None)

            if not title:
                len_old_title = obj.__class__.objects.filter(title__icontains=obj.title).count()
                title = f"{obj.title} ({len_old_title})" if len_old_title > 0 else obj.title

            new_obj = obj.clone_object(title=title)

            new_obj.save()
        link = reverse("admin:garpix_page_basecomponent_changelist")
        return HttpResponseRedirect(link)



from .base_page import BasePage  # noqa
from .base_list_page import BaseListPage  # noqa
from .base_search_page import BaseSearchPage  # noqa
from .components import * # noqa
from .settings import *  # noqa



from setuptools import setup, find_packages
from os import path

here = path.join(path.abspath(path.dirname(__file__)), 'garpix_page')

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='garpix_page',
    version='2.38.0',
    description='',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/garpixcms/garpix_page',
    author='Garpix LTD',
    author_email='info@garpix.com',
    license='MIT',
    packages=find_packages(exclude=['testproject', 'testproject.*']),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'Django >= 1.11',
        'django-polymorphic-tree-for-garpix-page >= 2.1.1',
        'django-modeltranslation >= 0.16.2',
        'django-multiurl >= 1.4.0',
        'djangorestframework >= 3.12.4',
        'garpix_utils >= 1.6.0',
        'django-tabbed-admin >= 1.0.4',
        'model-bakery >= 1.4.0',
        'garpix-admin-lock >= 1.0.0',
        'psycopg2-binary >=2.9.3'
    ],
)



from django.core.management.base import BaseCommand
from ...codegenerator import generate_page


class Command(BaseCommand):
    help = 'Start new page. Example: python3 backend/manage.py startpage --app=news --page=category --base=list'

    def add_arguments(self, parser):
        parser.add_argument('--app', type=str)
        parser.add_argument('--page', type=str)
        parser.add_argument('--base', type=str)

    def handle(self, *args, **options):
        app = options['app']
        page = options['page']
        base = options['base']
        generate_page(app, page, base)
        self.stdout.write(self.style.SUCCESS('Done'))



from rest_polymorphic.serializers import PolymorphicSerializer

from ..serializer import get_serializer
from garpix_page.utils.get_garpix_page_models import get_garpix_page_component_models


def get_components_serializers_list():
    serializers_list = {}
    for model in get_garpix_page_component_models():
        serializers_list.update({model: get_serializer(model)})
    return serializers_list


class ComponentSerializer(PolymorphicSerializer):
    model_serializer_mapping = get_components_serializers_list()



import os
from django.conf import settings
from django.contrib import admin

__all__ = ('GrapesJsAdminMixin', )

from django.urls import path

from garpix_page.views.get_template import GetTemplate
from garpix_page.views.upload import DgjsUpload


class GrapesJsAdminMixin(admin.ModelAdmin):
    change_form_template = 'grapesjs/change_form.html'

    def save_form(self, request, form, change):
        obj = super().save_form(request, form, change)
        css = request.POST.get('html_css')
        js = request.POST.get('html_js')
        field_name = 'html'
        filename_css = f'{settings.MEDIA_ROOT}/grapesjs/{field_name}_{obj.pk}_css.css'
        filename_js = f'{settings.MEDIA_ROOT}/grapesjs/{field_name}_{obj.pk}_js.js'
        os.makedirs(os.path.dirname(filename_css), exist_ok=True)
        with open(filename_css, "w") as f:
            f.write(css)

        os.makedirs(os.path.dirname(filename_js), exist_ok=True)
        with open(filename_js, "w") as f:
            f.write(js)
        return obj

    def get_urls(self):
        urls = super().get_urls()

        my_urls = [
            path('get_template/', GetTemplate.as_view(), name='dgjs_get_template'),
            path('dgjs_upload/', DgjsUpload.as_view(), name='dgjs_upload'),
        ]

        return my_urls + urls



from django.urls import path, re_path
from django.conf import settings
from django.contrib.sitemaps.views import sitemap
from garpix_page.views.page_api import PageApiView, PageApiListView
from garpix_page.views.robots import robots_txt
from garpix_page.views.sitemap import sitemap_view

app_name = 'garpix_page'

urlpatterns = [
    re_path(r'{}/page_models_list/$'.format(settings.API_URL), PageApiListView.as_view()),
    re_path(r'{}/page/(?P<slugs>.*)/$'.format(settings.API_URL), PageApiView.as_view()),
    re_path(r'{}/page/(?P<slugs>.*)$'.format(settings.API_URL), PageApiView.as_view()),
    path('sitemap.xml', sitemap, sitemap_view(), name='django.contrib.sitemaps.views.sitemap'),
    path('robots.txt', robots_txt),
]



from modeltranslation.translator import TranslationOptions, register
from garpix_page.models import SeoTemplate


@register(SeoTemplate)
class SeoTemplateTranslationOptions(TranslationOptions):
    fields = ('seo_title', 'seo_keywords', 'seo_description', 'seo_author')



import os
from django.conf import settings
from .constants import *


def snake_to_camel_case(text_snake):
    return '{}'.format(
        text_snake.title().replace('_', ''),
    )


def create_file(directory, file_name, file_content):
    path = os.path.join(directory, file_name)
    if not os.path.isfile(path):
        with open(path, 'w') as f:
            f.write(file_content)


def create_or_append_file(directory, file_name, file_content):
    path = os.path.join(directory, file_name)
    if not os.path.isfile(path):
        create_file(directory, file_name, file_content)
    else:
        with open(path, 'r') as f:
            lines = f.readlines()
            if file_content in lines:
                return
        with open(path, 'a') as f:
            f.write(file_content)


def generate_page(app, page, base):
    base_directory = os.path.abspath(os.path.join(settings.BASE_DIR))
    available_bases = ['page', 'list', 'search']
    if base not in available_bases:
        raise Exception(f'Неправильно выбран параметр base. Допустимые значения: {", ".join(available_bases)}')
    # create models
    directory = os.path.join(base_directory, os.path.dirname(FILE_NAME_MODEL.format(app=app, page=page)))
    os.makedirs(
        directory,
        exist_ok=True
    )
    # models page
    base_name = os.path.basename(FILE_NAME_MODEL.format(app=app, page=page))
    content = FILE_CONTENT_MODEL_DICT[base].format(app=app, page=page, page_capitalize=snake_to_camel_case(page))
    create_file(directory, base_name, content)
    # models init
    base_name = os.path.basename(FILE_NAME_MODEL_INIT.format(app=app, page=page))
    content = FILE_CONTENT_MODEL_INIT.format(page=page, page_capitalize=snake_to_camel_case(page))
    create_or_append_file(directory, base_name, content)

    print(f'Не забудьте поменять verbose_name и verbose_name_plural в файлах: backend/{FILE_NAME_MODEL.format(app=app, page=page)}')

    # create admin
    directory = os.path.join(base_directory, os.path.dirname(FILE_NAME_ADMIN.format(app=app, page=page)))
    os.makedirs(
        directory,
        exist_ok=True
    )
    # admin page
    base_name = os.path.basename(FILE_NAME_ADMIN.format(app=app, page=page))
    content = FILE_CONTENT_ADMIN.format(app=app, page=page, page_capitalize=snake_to_camel_case(page))
    create_file(directory, base_name, content)
    # admin init
    base_name = os.path.basename(FILE_NAME_ADMIN_INIT.format(app=app, page=page))
    content = FILE_CONTENT_ADMIN_INIT.format(page=page, page_capitalize=snake_to_camel_case(page))
    create_or_append_file(directory, base_name, content)

    # create translation
    directory = os.path.join(base_directory, os.path.dirname(FILE_NAME_TRANSLATION.format(app=app, page=page)))
    os.makedirs(
        directory,
        exist_ok=True
    )
    # translation page
    base_name = os.path.basename(FILE_NAME_TRANSLATION.format(app=app, page=page))
    content = FILE_CONTENT_TRANSLATION.format(app=app, page=page, page_capitalize=snake_to_camel_case(page))
    create_file(directory, base_name, content)
    # translation init
    base_name = os.path.basename(FILE_NAME_TRANSLATION_INIT.format(app=app, page=page))
    content = FILE_CONTENT_TRANSLATION_INIT.format(page=page, page_capitalize=snake_to_camel_case(page))
    create_or_append_file(directory, base_name, content)

    # create template
    directory = os.path.join(base_directory, os.path.dirname(FILE_NAME_TEMPLATE.format(page=page)))
    os.makedirs(
        directory,
        exist_ok=True
    )
    # template page
    base_name = os.path.basename(FILE_NAME_TEMPLATE.format(page=page))
    content = FILE_CONTENT_TEMPLATE_DICT[base]
    create_file(directory, base_name, content)

    # create app init
    directory = os.path.join(base_directory, os.path.dirname(FILE_NAME_APP_INIT.format(app=app)))
    os.makedirs(
        directory,
        exist_ok=True
    )
    # app init
    base_name = os.path.basename(FILE_NAME_APP_INIT.format(app=app))
    content = FILE_CONTENT_APP_INIT.format(app=app, app_capitalize=snake_to_camel_case(app))
    create_file(directory, base_name, content)

    # create app apps
    directory = os.path.join(base_directory, os.path.dirname(FILE_NAME_APPS.format(app=app)))
    os.makedirs(
        directory,
        exist_ok=True
    )
    # app apps
    base_name = os.path.basename(FILE_NAME_APPS.format(app=app))
    content = FILE_CONTENT_APPS.format(app=app, app_capitalize=snake_to_camel_case(app))
    create_file(directory, base_name, content)

    print(f'Не забудьте проверить, что verbose_name указан верно в файле: backend/{FILE_NAME_APPS.format(app=app)}')

    # create migrations
    directory = os.path.join(base_directory, os.path.dirname(FILE_NAME_MIGRATIONS_INIT.format(app=app)))
    os.makedirs(
        directory,
        exist_ok=True
    )
    # app apps
    base_name = os.path.basename(FILE_NAME_MIGRATIONS_INIT.format(app=app))
    create_file(directory, base_name, '')



from django.db.models.signals import post_delete
from django.dispatch import receiver

from garpix_page.cache import cache_service

from garpix_page.models import SeoTemplate


@receiver(post_delete, sender=SeoTemplate)
def clean_seo_cache(sender, *args, **kwargs):
    cache_service.clear_seo_data()



from django.conf import settings
from django.http import Http404

from garpix_page.mixins.views import PageViewMixin
from ..utils.get_garpix_page_models import get_garpix_page_models
from ..utils.check_redirect import check_redirect
from django.shortcuts import redirect
from django.views.generic import DetailView

from ..utils.get_languages import get_languages
from urllib.parse import urlencode

languages_list = get_languages()


class PageView(PageViewMixin, DetailView):
    def get_template_names(self):
        """
        Метод для получения темплейта для страницы.
        Структуру темплейтов можно посмотреть в модели page.models.Page.
        """
        if self.object is not None:
            return [self.object.get_template()]
        return []

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        if self.object is not None:
            context.update(self.object.get_context(self.request, object=self.object))
        return context

    def _get_home_page(self):
        for Model in get_garpix_page_models():
            home_page = Model.on_site.filter(slug='', is_active=True).first()
            if home_page is not None:
                return home_page
        raise Http404

    def get_object(self, queryset=None):
        """
        Метод для получения объекта страницы.
        Сравнивает текущий урл со slug, которые мы получаем из родительских страниц нашей страницы.
        """
        url = self.kwargs.get('url', None)
        # home pages
        if url is None or url == '':
            return self._get_home_page()

        obj = self.get_instance_by_slug(url, languages_list)

        return obj

    def _check_permissions(self, request):

        user = request.user

        if getattr(self.object, 'login_required', False):
            if not user.is_authenticated:
                return False
        if not self.object.has_permission_required(request):
            return False

        if getattr(self.object, 'query_parameters_required', None) is not None:
            request_get = set(request.GET.keys())
            parameters = set(self.object.query_parameters_required)
            if request_get != parameters:
                return False
        return True

    def get(self, request, *args, **kwargs):
        from django.shortcuts import render

        url = self.kwargs.get('url', None)

        if url and url[-1] == '/':
            parameters = f'?{urlencode(request.GET)}' if urlencode(request.GET) else ''
            return redirect(f"/{url.rstrip('/')}{parameters}")

        self.object = self.get_object()

        if not self.object:
            try:
                response = render(request, "404.html", context={})
                response.status_code = 404
                return response
            except Exception:
                raise Http404

        if not self._check_permissions(request):
            return redirect(settings.LOGIN_URL)

        context = self.get_context_data(object=self.object)
        redir = check_redirect(request, context)
        if redir:
            return redirect(redir)
        return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        user = request.user
        if getattr(self.object, 'login_required', False):
            if not user.is_authenticated:
                return redirect(settings.LOGIN_URL)
        if not self.object.has_permission_required(request):
            return redirect(settings.LOGIN_URL)
        context = self.get_context_data(object=self.object)
        redir = check_redirect(request, context)
        if redir:
            return redirect(redir)
        return self.render_to_response(context)



from django import forms

from garpix_page.forms import GrapesJsWidget
from garpix_page.settings import BASE, GRAPESJS_DEFAULT_HTML, REDACTOR_CONFIG
from garpix_page.utils import apply_string_handling

__all__ = (
    'GrapesJsField',
)


class GrapesJsField(forms.CharField):
    '''
    Form field with support grapesjs.
    '''

    widget = GrapesJsWidget

    def __init__(self, default_html=GRAPESJS_DEFAULT_HTML, html_name_init_conf=REDACTOR_CONFIG[BASE],
                 apply_django_tag=False, validate_tags=False, template_choices=None, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.widget.default_html = default_html
        self.widget.html_name_init_conf = html_name_init_conf
        self.widget.apply_django_tag = apply_django_tag
        self.widget.template_choices = template_choices
        self.validate_tags = validate_tags

    def validate(self, value):
        super().validate(value)
        # TODO: check the field
        # if self.validate_tags:

    def clean(self, value):
        value = apply_string_handling(value, 'apply_tag_save')

        return super().clean(value)



from rest_framework import status
from django.utils.translation import activate
from rest_framework import views
from rest_framework.response import Response
import django.apps
from django.utils.module_loading import import_string
from django.conf import settings

from garpix_page.mixins.views import PageViewMixin
from ..serializers.serializer import get_serializer
from ..utils.get_languages import get_languages

model_list = []
for model in django.apps.apps.get_models():
    try:
        if model.is_for_page_view():
            model_list.append(model)
    except:  # noqa
        pass


languages_list = get_languages()


class PageApiView(PageViewMixin, views.APIView):

    @staticmethod
    def get_error_page_response_data(page, request, page_name):
        return {
            'page_model': page_name,
            'init_state': {
                "object": None,
                "global": import_string(settings.GARPIX_PAGE_GLOBAL_CONTEXT)(request, page)
            }
        }

    def check_errors(self, page, request):
        if page is None:
            return Response(self.get_error_page_response_data(page, request, 'Page404'),
                            status=status.HTTP_404_NOT_FOUND)

        if getattr(page, 'login_required', False):
            if not request.user.is_authenticated:
                return Response(self.get_error_page_response_data(page, request, 'Page401'), status=status.HTTP_401_UNAUTHORIZED)

        if not page.has_permission_required(request):
            return Response(self.get_error_page_response_data(page, request, 'Page403'), status=status.HTTP_403_FORBIDDEN)

        if getattr(page, 'query_parameters_required', None) is not None:
            request_get = set(request.GET.keys())
            parameters = set(page.query_parameters_required)
            if request_get != parameters:
                return Response(self.get_error_page_response_data(page, request, 'Page404'), status=status.HTTP_404_NOT_FOUND)

        return None

    def get_object(self, slugs):

        obj = self.get_instance_by_slug(slugs, languages_list)
        return obj

    def get(self, request, slugs):  # noqa

        language = languages_list[0]
        if 'HTTP_ACCEPT_LANGUAGE' in request.META and request.META['HTTP_ACCEPT_LANGUAGE'] in languages_list:
            language = request.META['HTTP_ACCEPT_LANGUAGE']
        activate(language)

        page = self.get_object(slugs)

        errors = self.check_errors(page, request)
        if errors is not None:
            return errors

        page_context = page.get_context(request, object=page, user=request.user, api=True)
        for k, v in page_context.items():
            if hasattr(v, 'is_for_page_view'):
                model_serializer_class = get_serializer(v.__class__)
                page_context[k] = model_serializer_class(v, context={"request": request}).data
        if 'paginated_object_list' in page_context:
            try:
                serializer_class = get_serializer(page_context['paginated_object_list'][0].__class__)
                page_context['paginated_object_list'] = serializer_class(page_context['paginated_object_list'],
                                                                         context={"request": request}, many=True).data
            except Exception:
                page_context['paginated_object_list'] = list(
                    {'id': x.id, 'title': x.title, 'get_absolute_url': x.get_absolute_url()} for x in
                    page_context['paginated_object_list'])
        if 'paginator' in page_context:
            page_context['num_pages'] = page_context['paginator'].num_pages
            page_context['per_page'] = page_context['paginator'].per_page
            page_context.pop('paginator')

        page_context['global'] = import_string(settings.GARPIX_PAGE_GLOBAL_CONTEXT)(request, page)
        page_context['object'].update({
            'components': page.get_components_context(request, api=True)
        })
        data = {
            'page_model': page.__class__.__name__,
            'init_state': page_context,
        }
        return Response(data)


class PageApiListView(views.APIView):
    def get(self, request):  # noqa
        from garpix_page.utils.get_garpix_page_models import get_garpix_page_models
        models_list = get_garpix_page_models()
        data = {}
        for model in models_list:
            data.update({model.__name__: model._meta.verbose_name})
        return Response(data)
